using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Amazon;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.DataModel;
using Amazon.Lambda.Core;
using Amazon.Lambda.DynamoDBEvents;
using Amazon.Runtime;
using AwS.APG.CQRSES.EventSourceCustomer.Models;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace AWS.APG.CQRSES.EventSourceCustomer
{
    public class Function
    {

        private static AmazonDynamoDBClient _client;
        private readonly string _accessKey;
        private readonly string _secretKey;


        /// <summary>
        /// For testing use only.
        /// </summary>
        public Function(string accessKey,
            string secretKey)
        {
            _accessKey = accessKey;
            _secretKey = secretKey;

            Initialize();
        }

        public Function()
        {
            //-- get keys and service url from enviornment variable
            _accessKey = Environment.GetEnvironmentVariable("AccessKey");
            _secretKey = Environment.GetEnvironmentVariable("SecretKey");

            Initialize();
        }

        private void Initialize()
        {
            _client = new AmazonDynamoDBClient(
                new BasicAWSCredentials(_accessKey, _secretKey),
                new AmazonDynamoDBConfig
                {
                    //ServiceURL = _serviceUrl,
                    RegionEndpoint = RegionEndpoint.USEast1
                });
        }

        public async Task FunctionHandler(DynamoDBEvent dynamoEvent, ILambdaContext context)
        {
            context.Logger.LogLine($"Beginning to process {dynamoEvent.Records.Count} records...");

            await UpdateCustomerList(dynamoEvent, context);

            context.Logger.LogLine("Stream processing complete.");
        }


        private static async Task UpdateCustomerList(DynamoDBEvent dynamoEvent, ILambdaContext context)
        {
            DynamoDBContext dbContext = new DynamoDBContext(_client);

            foreach (var record in dynamoEvent.Records)
            {
                context.Logger.LogLine($"Event ID: {record.EventID}");
                context.Logger.LogLine($"Event Name: {record.EventName}");

                if (record.EventName == "INSERT")
                {
                    //-- get the newly added reward from the stream
                    var customerNew = new Customer(record.Dynamodb.NewImage);
                    
                    //-- get the customer
                    var customerList = await dbContext.LoadAsync<CustomerList>(customerNew.Id);

                    if (customerList == null)
                    {
                        //-- insert new customer in query table
                        await dbContext.SaveAsync<CustomerList>(new CustomerList()
                        { 
                            Id = customerNew.Id,
                            Lastname = customerNew.Lastname,
                            Firstname = customerNew.Firstname,
                            VIP = customerNew.VIP
                        });
                    }
                }

                else if (record.EventName == "MODIFY")
                {
                    //-- get the newly added reward from the stream
                    var customerNew = new Customer(record.Dynamodb.NewImage);
                    var customerOld = new Customer(record.Dynamodb.OldImage);

                    //-- get the customer
                    var customerList = await dbContext.LoadAsync<CustomerList>(customerNew.Id);

                    if (customerList != null)
                    {
                        if (!customerNew.Firstname.Equals(customerOld.Firstname))
                        {
                            customerList.Firstname = customerNew.Firstname;                            
                        }
                        
                        if (!customerNew.Lastname.Equals(customerOld.Lastname))
                        {
                            customerList.Lastname = customerNew.Lastname;

                        }

                        if (!customerNew.VIP.Equals(customerOld.VIP))
                        {
                            customerList.VIP = customerNew.VIP;

                        }

                        //-- update customer in customer query table
                        await dbContext.SaveAsync<CustomerList>(customerList);
                    }
                }

                else if (record.EventName == "REMOVE")
                {
                    //-- get the newly added reward from the stream
                    var customerOld = new Customer(record.Dynamodb.OldImage);

                    //-- remove customer from customer query table
                    await dbContext.DeleteAsync<CustomerList>(customerOld.Id);
                    
                }
            }
        }
    }
}
