﻿using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.Model;
using Amazon.Lambda.DynamoDBEvents;
using Amazon.Lambda.TestUtilities;
using AWS.APG.CQRSES.EventSourceCustomer;
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using System.Linq;
using Xunit;

using CustomerListQuery = AWS.APG.CQRSES.QueryCustomerListLambda;

namespace AWS.APG.CQRSES.EventSourceLambda.Tests
{
    public class CustomerEventFunctionTest : BaseFunctionTest
    {
        /// <summary>
        /// Mock of dynamod db event stream. 
        /// </summary>
        private DynamoDBEvent MockInsertDynamoDBEvent
        {
            get
            {
                //-- define the event definintion to simulate the NEW_IMAGE stream event
                return new DynamoDBEvent
                {
                    Records = new List<DynamoDBEvent.DynamodbStreamRecord>
                    {
                        new DynamoDBEvent.DynamodbStreamRecord
                        {
                            AwsRegion = "us-east-1",
                            EventName="INSERT",
                            Dynamodb = new StreamRecord
                            {

                                ApproximateCreationDateTime = DateTime.Now,
                                Keys = new Dictionary<string, AttributeValue> { {"Id", new AttributeValue { N = JohnDoeCustomerId.ToString() } } },
                                OldImage = null,
                                NewImage = new Dictionary<string, AttributeValue>
                                {
                                    { "Id", new AttributeValue { N = JohnDoeCustomerId.ToString() }},
                                    { "Firstname", new AttributeValue { S = "John" }},  
                                    { "Lastname", new AttributeValue { S = "Do" }}, 
                                    { "CompanyName", new AttributeValue { S = "Amazon Web Service"}},
                                    { "Address", new AttributeValue { S = "Singapore, Singapore"}},
                                    { "VIP", new AttributeValue { N = "1"}}
                                },
                                StreamViewType = StreamViewType.NEW_AND_OLD_IMAGES,


                            }
                        }
                    }
                };

            }
        }


        private DynamoDBEvent MockModifyDynamoDBEvent
        {
            get
            {
                //-- define the event definintion to simulate the NEW_IMAGE stream event
                return new DynamoDBEvent
                {
                    Records = new List<DynamoDBEvent.DynamodbStreamRecord>
                    {
                        new DynamoDBEvent.DynamodbStreamRecord
                        {
                            AwsRegion = "us-east-1",
                            EventName="MODIFY",
                            Dynamodb = new StreamRecord
                            {

                                ApproximateCreationDateTime = DateTime.Now,
                                Keys = new Dictionary<string, AttributeValue> { {"Id", new AttributeValue { N = JohnDoeCustomerId.ToString() } } },
                                OldImage = new Dictionary<string, AttributeValue>
                                {
                                    { "Id", new AttributeValue { N = JohnDoeCustomerId.ToString() }},
                                    { "Firstname", new AttributeValue { S = "John" }},
                                    { "Lastname", new AttributeValue { S = "Do" }},
                                    { "CompanyName", new AttributeValue { S = "Amazon Web Service"}},
                                    { "Address", new AttributeValue { S = "Singapore, Singapore"}},
                                    { "VIP", new AttributeValue { N = "1"}}
                                },
                                NewImage = new Dictionary<string, AttributeValue>
                                {
                                    { "Id", new AttributeValue { N = "10" }},
                                    { "Firstname", new AttributeValue { S = "John" }},
                                    { "Lastname", new AttributeValue { S = "Doe" }},
                                    { "CompanyName", new AttributeValue { S = "Blue Origin"}},
                                    { "Address", new AttributeValue { S = "Texas, United States"}},
                                    { "VIP", new AttributeValue { N = "1"}}
                                },
                                StreamViewType = StreamViewType.NEW_AND_OLD_IMAGES,

                            }
                        }
                    }
                };

            }
        }


        private DynamoDBEvent MockRemoveDynamoDBEvent
        {
            get
            {
                //-- define the event definintion to simulate the NEW_IMAGE stream event
                return new DynamoDBEvent
                {
                    Records = new List<DynamoDBEvent.DynamodbStreamRecord>
                    {
                        new DynamoDBEvent.DynamodbStreamRecord
                        {
                            AwsRegion = "us-east-1",
                            EventName="REMOVE",
                            Dynamodb = new StreamRecord
                            {

                                ApproximateCreationDateTime = DateTime.Now,
                                Keys = new Dictionary<string, AttributeValue> { {"Id", new AttributeValue { N = JohnDoeCustomerId.ToString() } } },
                                OldImage = new Dictionary<string, AttributeValue>
                                {
                                    { "Id", new AttributeValue { N = JohnDoeCustomerId.ToString() }},
                                    { "Firstname", new AttributeValue { S = "John" }},
                                    { "Lastname", new AttributeValue { S = "Doe" }},
                                    { "CompanyName", new AttributeValue { S = "Amazon Web Service"}},
                                    { "Address", new AttributeValue { S = "Singapore, Singapore"}},
                                    { "VIP", new AttributeValue { N = "1"}}
                                },
                                NewImage = null,
                                StreamViewType = StreamViewType.NEW_AND_OLD_IMAGES,

                            }
                        }
                    }
                };

            }
        }


        /// <summary>
        /// This is to simulate the dynamo event trigger.
        /// What we expect here, is when the cqurses-reward-cmd table trigger event it will update the cqrses-reward-query customer reward aggregated table customer points.
        /// </summary>
        [Fact]
        public async Task DynamoDBEventTestFunction()
        {

            var context = new TestLambdaContext();
            var eventFunction = new Function(AccessKey, SecretKey);
            var queryCustomer = new CustomerListQuery.Function(AccessKey, SecretKey);


            //-- trigger INSERT event
            await eventFunction.FunctionHandler(MockInsertDynamoDBEvent, context);
            
            var customerList = await queryCustomer.FunctionHandler(context);
            var customer = customerList.FirstOrDefault(c => c.Id.Equals(JohnDoeCustomerId));
            Assert.NotNull(customer);
            Assert.Equal("Do", customer.Lastname);


            //-- trigger MODIFY event
            await eventFunction.FunctionHandler(MockModifyDynamoDBEvent, context);
            
            customerList = await queryCustomer.FunctionHandler(context);
            customer = customerList.FirstOrDefault(c => c.Id.Equals(JohnDoeCustomerId));
            Assert.NotNull(customer);
            Assert.Equal("Doe", customer.Lastname);


            //-- trigger REMOVE event
            await eventFunction.FunctionHandler(MockRemoveDynamoDBEvent, context);
            
            customerList = await queryCustomer.FunctionHandler(context);
            customer = customerList.FirstOrDefault(c => c.Id.Equals(JohnDoeCustomerId));
            Assert.Null(customer);


            var testLogger = context.Logger as TestLambdaLogger;
            Assert.Contains("Stream processing complete", testLogger.Buffer.ToString());
        }
    }
}
