using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using Xunit;

using Amazon;
using Amazon.Lambda.Core;
using Amazon.Lambda.DynamoDBEvents;
using Amazon.Lambda.TestUtilities;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.Model;

using CustomerRewardQuery = AWS.APG.CQRSES.QueryRewardLambda;
using AWS.APG.CQRSES.EventSourceReward;

namespace AWS.APG.CQRSES.EventSourceLambda.Tests
{
    public class RewardEventFunctionTest : BaseFunctionTest
    {
        
        /// <summary>
        /// Points to credit to customer rewards. Just negate the value if you want to deduct points.
        /// </summary>
        private int Points
        {
            get { return 10; }
        }

        /// <summary>
        /// Mock of dynamod db event stream. 
        /// </summary>
        private DynamoDBEvent MockDynamoEvent
        {
            get
            {
                //-- define the event definintion to simulate the NEW_IMAGE stream event
                return new DynamoDBEvent
                {
                    Records = new List<DynamoDBEvent.DynamodbStreamRecord>
                    {
                        new DynamoDBEvent.DynamodbStreamRecord
                        {
                            AwsRegion = "us-east-1",
                            EventName="INSERT",
                            Dynamodb = new StreamRecord
                            {

                                ApproximateCreationDateTime = DateTime.Now,
                                Keys = new Dictionary<string, AttributeValue> { {"Id", new AttributeValue { N = "2" } } },
                                NewImage = new Dictionary<string, AttributeValue>
                                {
                                    { "Id", new AttributeValue { N = "1001" } },
                                    { "CustomerId", new AttributeValue { N = ShirleyRodriguezCustomerId.ToString() } },  //-- Shirley Rodriquez customer
                                    { "Points", new AttributeValue { N = Points.ToString() } },      //-- points
                                    { "Timestamp", new AttributeValue { S = DateTime.UtcNow.ToString("s")} }
                                },
                                StreamViewType = StreamViewType.NEW_IMAGE,


                            }
                        }
                    }
                };

            }
        }

        /// <summary>
        /// This is to simulate the dynamo event trigger.
        /// What we expect here, is when the cqurses-reward-cmd table trigger event it will update the cqrses-reward-query customer reward aggregated table customer points.
        /// </summary>
        [Fact]
        public async Task DynamoDBEventTestFunction()
        {           

            var context = new TestLambdaContext();
            var eventFunction = new Function(AccessKey, SecretKey);
            var rewardFunction = new CustomerRewardQuery.Function(AccessKey, SecretKey);

            //-- get the current reward points. We will use this to assert and check if the case is valid.
            var oldReward = await rewardFunction.FunctionHandler(ShirleyRodriguezCustomerId, context);

            //-- trigger the Event handler, this should add 10 points to Shirley's reward points
            await eventFunction.FunctionHandler(MockDynamoEvent, context);

            //-- check the aggregated customer reward after the event.
            var newReward = await rewardFunction.FunctionHandler(ShirleyRodriguezCustomerId, context);

            Assert.Equal(oldReward.Points + Points, newReward.Points);
            var testLogger = context.Logger as TestLambdaLogger;
			Assert.Contains("Stream processing complete", testLogger.Buffer.ToString());
        }  
    }
}
